/* Argument checker for ls */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "ptwrapper_o.h"
#include "debug.h"

char *progname = "ls";
/*D

  ptls - SUT ls

Description:

ptls lists files on specified hosts.

Command Line Arguments:

ptls [-all|-m <machine_file>|-M <machine_list>] [-qcrCRdfFgisltLumxna1oApbh] [<file1> <file2> <file3>...]

.N machine_args
. -qcrCRdfFgisltLumxna1oApb - same as for ls(1)
. -h - print the hostname as a header for the output from each host
- [<file1> <file2> <file3>...] - the list of files to list

Environment Variables:

.N machine_envs

Examples:

To list the contents of the current directory on the machines specified in the 
file "destination_machines" use

.vb
ptls -m destination_machines
.ve

To do the same listing, specifying the machines on the command line, use

.vb
ptls -M "host1 host2 host3"
.ve

To list all files in the current directory with the hostname preceeding each
line printed, use

.vb
ptls -m destination_machines -h
.ve

.seealso ls(1)
  
 D*/

int check_args(int argc, char *argv[], char *cmd)
{
	int len;
    int i, c;
    char opt[4];

	snprintf(cmd, MAX_CMD_LEN, "%s ", progname);
	len = strlen(cmd);

    while ((c = getopt(argc, argv, "qcrCRdfFgisltLumxna1oApbh")) != -1){
        switch (c){
            case 'q':
            case 'c':
            case 'r':
            case 'C':
            case 'R':
            case 'd':
            case 'f':
            case 'F':
            case 'g':
            case 'i':
            case 's':
            case 'l':
            case 't':
            case 'L':
            case 'u':
            case 'm':
            case 'x':
            case 'n':
            case 'a':
            case '1':
            case 'o':
            case 'A':
            case 'p':
            case 'b':
                len += 3;
                if (len >= MAX_CMD_LEN)
                    return -2;
                sprintf(opt, "-%c ", c);
                strcat(cmd, opt);
                break;
            case 'h':
                add_hostname = 1;
                break;
            default:
                return -1;
        }
    }

    for (i = optind; i < argc; i++){
        len += strlen(argv[i]) + 1;
        if (len >= MAX_CMD_LEN)
            return -2;
		strcat(cmd, argv[i]);
        strcat(cmd, " ");
    }
    
    pt_debug("the command is '%s'\n", cmd);

    return 0;
}
