/* Argument checker for mkdir */
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "debug.h"

char *progname = "mkdir";

/*D

  ptmkdir - Parallel mkdir

Description:

ptmkdir makes directories on specified hosts.

Command Line Arguments:

ptmkdir [-m <machine_file>|-M <machine_list>] [-p] <dir1> [<dir2> <dir3>...]

.N machine_args

. -p - create any parent directories if they do not exist
- <dir1> [<dir2> <dir3>...] - the list of directories to create

Environment Variables:

.N machine_envs

Examples:

To create the directory "mydir" on the machines specified in the file 
"destination_machines" use

.vb
ptmkdir -m destination_machines mydir
.ve

To make the same directory, specifying the machines on the command line, use

.vb
ptmkdir -M "host1 host2 host3" mydir
.ve

To create the directory "mydir/mysubdir", ensuring that mydir exists, use 

.vb
ptmkdir -m destination_machines -p mydir/mysubdir
.ve

Notes:

The mkdir(1) option '-m' is not supported because it conflicts with the 
machine file option.  To get the same result, use ptmkdir, followed by ptchmod(1).

.seealso mkdir(1) ptchmod(1)
  
 D*/
int check_args(int argc, char *argv[])
{
    int c;

    while ((c = getopt(argc, argv, "p")) != -1) {
        switch (c) {
            case 'p':
                continue;
            default:
                return -1;
        }
    }

    pt_debug("optind = %d, argc = %d\n", optind, argc);
    if (argc - optind < 1)
        return -1;

    return 0;
}
