/* Argument checker for rmdir */
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

char *progname = "rmdir";

/*D

  ptrmdir - SUT rmdir

Description:

ptrmdir removes directories on specified hosts.

Command Line Arguments:

ptrmdir [-m <machine_file>|-M <machine_list>] [-p] <dir1> [<dir2> <dir3>...]

.N machine_args

. -p - remove any parent directories if they exist and are empty after
removing the child directory
- <dir1> [<dir2> <dir3>...] - the list of directories to remove

Environment Variables:

.N machine_envs

Examples:

To remove the directory "mydir" on the machines specified in the file 
"destination_machines" use

.vb
ptrmdir -m destination_machines mydir
.ve

To remove the same directory, specifying the machines on the command line, use

.vb
ptrmdir -M "host1 host2 host3" mydir
.ve

To remove the directory "mydir/mysubdir", also removing mydir if it is empty
after removing mysubdir, use 

.vb
ptrmdir -m destination_machines -p mydir/mysubdir
.ve

.seealso rmdir(1)
  
 D*/
int check_args(int argc, char *argv[])
{
    int c;

    while ((c = getopt(argc, argv, "p")) != -1) {
        switch (c) {
            case 'p':
                continue;
            default:
                return -1;
        }
    }

    if (argc - optind < 1)
        return -1;

    return 0;
}
