#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <gtk/gtk.h>

static char fifobuf[32];
static char *fifoname;
static int input_is_fifo = 0;
static int delete_fifo = 0;
static gint show_host = 1;
static gint maintable_size = 5;

#define MAX_CELL            (maintable_size*maintable_size)

#define CELL_LEFT(c)        ((c) % maintable_size)
#define CELL_RIGHT(c)       (CELL_LEFT(c) + 1)
#define CELL_TOP(c)         ((c) / maintable_size)
#define CELL_BOTTOM(c)      (CELL_TOP(c) + 1)

#define BUF_SIZE	1024

static char *title = "ptdisp";
static GtkWidget *mainwindow, *maintable, *logtext, *logtext_hbox;
static GtkTooltips *button_tips;
static char host_cmd[BUF_SIZE] = "xterm -e ssh %s";

static void finalize(GtkWidget *widget, gpointer data)
{
    if (delete_fifo)
        unlink(fifoname);
    gtk_main_quit();
}

static void dialog_close(GtkWidget *widget, GtkWidget *dialog)
{
    gtk_grab_remove(dialog);
    gtk_widget_destroy(dialog);
}

static void show_about(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog, *label, *button;
    static char about_text[] = 
        "ptdisp is a generic display frontend for use with\n"
        "SUT - Scalable Unix Tools";

    dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dialog), "About ptdisp");
    gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
    button = gtk_button_new_with_label("Close");

    gtk_signal_connect(GTK_OBJECT(button), "clicked",
            GTK_SIGNAL_FUNC(dialog_close), dialog);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);

    label = gtk_label_new(about_text);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
    gtk_widget_show_all(dialog);
}

static void set_host_cmd(GtkWidget *widget, GtkEntry *entry)
{
    strncpy(host_cmd, gtk_entry_get_text(entry), BUF_SIZE);
}

static void configure(GtkWidget *widget, gpointer data)
{
    GtkWidget *dialog, *label, *button, *entry;

    dialog = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(dialog), "Configure");
    gtk_container_set_border_width(GTK_CONTAINER(dialog), 10);
    button = gtk_button_new_with_label("Close");
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);

    label = gtk_label_new("Enter command to run when a host button is pressed:");
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);

    entry = gtk_entry_new_with_max_length(BUF_SIZE/2);
    gtk_entry_set_text(GTK_ENTRY(entry), host_cmd);
    gtk_editable_select_region(GTK_EDITABLE(entry), 0, -1);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry, TRUE, TRUE, 0);

    gtk_signal_connect(GTK_OBJECT(button), "clicked",
            GTK_SIGNAL_FUNC(set_host_cmd), entry);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
            GTK_SIGNAL_FUNC(dialog_close), dialog);

    gtk_widget_show_all(dialog);
}

static GtkItemFactoryEntry menuitems[] = {
    {
        "/_File",
        NULL,
        NULL,
        0,
        "<Branch>"
    },
    {
        "/File/_Configure...", 
        "<alt>C", 
        (GtkItemFactoryCallback)configure, 
        0, 
        NULL
    },
    {
        "/File/-",
        NULL,
        NULL,
        0, 
        "<Separator>"
    },
    {
        "/File/_Quit", 
        "<alt>Q", 
        (GtkItemFactoryCallback)finalize, 
        0, 
        NULL
    },
    {
        "/_Help", 
        NULL, 
        NULL, 
        0, 
        "<LastBranch>"
    },
    {
        "/_Help/About", 
        NULL, 
        (GtkItemFactoryCallback)show_about, 
        0, 
        NULL
    },
};

static int init_mainwindow()
{
    GtkWidget *vbox, *menubar, *vscrollbar, *scrolledwin;
    GtkWidget *paned;
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;
    gint nomenuitems = sizeof(menuitems) / sizeof(menuitems[0]);

    /* Setup the main window */
    mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event", 
            GTK_SIGNAL_FUNC(finalize), NULL);
    gtk_window_set_title(GTK_WINDOW(mainwindow), title);
    gtk_window_set_policy(GTK_WINDOW(mainwindow), TRUE, TRUE, TRUE);
    gtk_window_set_default_size(GTK_WINDOW(mainwindow), 400, 300);
    gtk_widget_set_uposition(mainwindow, 0, 0);

    /* Make and insert the main vbox */
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(mainwindow), vbox);

    /* Create the menu from the item factory */
    accel_group = gtk_accel_group_new();
    item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
            accel_group);
    gtk_item_factory_create_items(item_factory, nomenuitems, menuitems, NULL);
    gtk_window_add_accel_group(GTK_WINDOW(mainwindow), accel_group);

    menubar = gtk_item_factory_get_widget(item_factory, "<main>");
    gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);

    /* Pack the split pane */
    paned = gtk_vpaned_new();
    gtk_box_pack_start(GTK_BOX(vbox), paned, TRUE, TRUE, 0);

    /* Create and pack the main table */
    maintable = gtk_table_new(maintable_size, maintable_size, TRUE);
    scrolledwin = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolledwin),
            maintable);
    gtk_paned_add1(GTK_PANED(paned), scrolledwin);

    /* Create and pack the log pane */
    logtext_hbox = gtk_hbox_new(FALSE, 0);
    gtk_paned_add2(GTK_PANED(paned), logtext_hbox);

    logtext = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(logtext), FALSE);
    gtk_box_pack_start(GTK_BOX(logtext_hbox), logtext, TRUE, TRUE, 0);

    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(logtext)->vadj);
    gtk_box_pack_start(GTK_BOX(logtext_hbox), vscrollbar, FALSE, FALSE, 0);

    /* Show everything */
    gtk_widget_show_all(mainwindow);

    /* Make button tool tips */
    button_tips = gtk_tooltips_new();
    gtk_tooltips_set_delay(button_tips, 0);

    return 0;
}

void grow_maintable()
{
    maintable_size++;
    gtk_table_resize(GTK_TABLE(maintable), maintable_size, maintable_size);
}

void add_cell(GtkWidget *cell, char *name) 
{
    struct cell {
        GtkWidget *widget;
        int left, right, top, bottom;
    } *tmp;
    static GHashTable *names = NULL;
    static GSList *freecells = NULL;
    static int next_cell = 0;
    int i, j, pos;
    int left, right, top, bottom;

    if (names == NULL) {
        names = g_hash_table_new(g_str_hash, g_str_equal);
        if (names == NULL) {
            perror("Unable to create cell hash table");
            gtk_main_quit();
        }
        for (i = 0; i < maintable_size; i++) {
            for (j = 0; j < maintable_size; j++) {
                freecells = g_slist_append(freecells, 
                        GINT_TO_POINTER((i*maintable_size) + j));
            }
        }
    }

    tmp = g_hash_table_lookup(names, name);
    if (tmp == NULL) {
        GSList *old;

        if (freecells == NULL) {
            for (i = 0; i < maintable_size; i++) {
                freecells = g_slist_append(freecells, 
                        GINT_TO_POINTER(maintable_size * (maintable_size + 1) + i));
            }
            for (i = 0; i < maintable_size + 1; i++) {
                freecells = g_slist_append(freecells, 
                        GINT_TO_POINTER(maintable_size+((maintable_size + 1) * i)));
            }
            grow_maintable();
        }
        pos = GPOINTER_TO_INT(freecells->data);
        old = freecells;
        freecells = g_slist_remove_link(freecells, freecells);
        g_slist_free_1(old);

        tmp = malloc(sizeof(struct cell));
        if (tmp == NULL) {
            perror("Unable to create cell");
            gtk_main_quit();
        }
        left = tmp->left = CELL_LEFT(pos);
        right = tmp->right = CELL_RIGHT(pos);
        top = tmp->top = CELL_TOP(pos);
        bottom = tmp->bottom = CELL_BOTTOM(pos);
        tmp->widget = cell;

        g_hash_table_insert(names, name, tmp);
    } else {
        gtk_widget_destroy(tmp->widget);
        tmp->widget = cell;

        left = tmp->left;
        right = tmp->right;
        top = tmp->top;
        bottom = tmp->bottom;
    }

    gtk_table_attach_defaults(GTK_TABLE(maintable), cell, left, right, top, bottom);

    gtk_widget_show_all(cell);
    next_cell++;
}

static void write_status(char *str)
{
    gtk_text_insert(GTK_TEXT(logtext), NULL, NULL, NULL, str, -1);
}

static void host_button_clicked(GtkWidget *button, char *hostname)
{
    int pid, status;
    char buf[BUF_SIZE];
    char msg[BUF_SIZE];

    g_snprintf(buf, BUF_SIZE, host_cmd, hostname);

    /* Execute the host command in the background */

    pid = fork();
    if (pid < 0) {
        g_snprintf(msg, BUF_SIZE, "Unable to execute '%s'\n", buf);
        write_status(msg);
        return;
    }

    if (pid == 0) {
        pid = vfork();
        if (pid < 0)
            _exit(1);
        if (pid == 0) {
            execlp("/bin/sh", "/bin/sh", "-c", buf, NULL);
            _exit(1);
        } 
        _exit(0);
    } else {
        wait(&status);
    }

    if (!WIFEXITED(status)) {
        g_snprintf(msg, BUF_SIZE, "Unable to execute '%s'\n", buf);
        write_status(msg);
    }
}

static void draw_legend(char *desc)
{
    static GtkWidget *legend_window = NULL;
    static GtkWidget *vbox = NULL;
    char text[BUF_SIZE], fg[BUF_SIZE], bg[BUF_SIZE], status[BUF_SIZE];
    int n = 0;

    if (legend_window == NULL) {
        legend_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_window_set_title(GTK_WINDOW(legend_window), "Legend");
        gtk_signal_connect(GTK_OBJECT(legend_window), "destroy",
                GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
    } else {
        gtk_container_remove(GTK_CONTAINER(legend_window), vbox);
        gtk_widget_destroy(vbox);
    }

    vbox = gtk_vbox_new(TRUE, 0);
    gtk_container_add(GTK_CONTAINER(legend_window), vbox);

    while (1) {
        GtkStyle *style;
        GtkWidget *button, *label;

        desc += n;

        if (sscanf(desc, "%s %s %s %n", text, fg, bg, &n) < 3)
            break;

        button = gtk_button_new();
        gtk_widget_set_state(button, GTK_STATE_INSENSITIVE);
        label = gtk_label_new(text);
        gtk_widget_set_state(label, GTK_STATE_INSENSITIVE);
        gtk_container_add(GTK_CONTAINER(button), label);

        style = gtk_style_copy(gtk_widget_get_style(label));
        if (gdk_color_parse(fg,&(style->fg[GTK_STATE_INSENSITIVE])) != TRUE){
            g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", fg);
            write_status(status);
            gtk_widget_destroy(button);
            return;
        }
        gtk_widget_set_style(label, style);
        gtk_widget_show(label);

        style = gtk_style_copy(gtk_widget_get_style(button));
        if (gdk_color_parse(bg, &(style->bg[GTK_STATE_INSENSITIVE])) != TRUE) {
            g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", bg);
            write_status(status);
            gtk_widget_destroy(button);
            return;
        }

        gtk_widget_set_style(button, style);
        gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
        gtk_widget_show_all(button);
    }

    gtk_widget_show_all(legend_window);
}

static int input_idx = 0;

static void handle_input(gpointer data, gint source, GdkInputCondition condition)
{
    FILE *f;
    char *tmp, *host;
    char buf[BUF_SIZE];
    char fmt[BUF_SIZE];
    char fg[BUF_SIZE], bg[BUF_SIZE];
    char status[BUF_SIZE];
    int argstart;
    int value;
    GtkStyle *style;

    if (condition != GDK_INPUT_READ)
        return;

    f = fdopen(source, "r");
    if (f == NULL)
        return;

    while (fgets(buf, BUF_SIZE, f) != NULL) {
        g_snprintf(status, BUF_SIZE, "Received: %s", buf);
        write_status(status);

        tmp = strchr(buf, ':');
        if (tmp == NULL)
            continue;

        *tmp = '\0';
        tmp++;

        if (strcmp(buf, "<LEGEND>") == 0){
            draw_legend(tmp);
            continue;
        }

        host = g_strdup(buf);
        if (host == NULL) {
            g_snprintf(status, BUF_SIZE, "Unable to allocate hostname: %s",
                    g_strerror(errno));
            write_status(status);
            continue;
        }

        if (sscanf(tmp, "%s %n", fmt, &argstart) < 1) {
            g_snprintf(status, BUF_SIZE, "No format given for '%s'\n", host);
            write_status(status);
            free(host);
            continue;
        }

        if (strcasecmp(fmt, "percentage") == 0) {
            int noargs;
            double percent;
            GtkWidget *vbox, *button, *label, *progress;

            noargs = sscanf(tmp + argstart, "%lf %s %s", &percent, fg, bg);
            if (noargs < 1) {
                g_snprintf(status, BUF_SIZE, "No percentage given\n");
                write_status(status);
                free(host);
                continue;
            }

            if (percent > 100.0 || percent < 0.0) {
                g_snprintf(status, BUF_SIZE, 
                        "Invalid percentage '%f'\n", percent);
                write_status(status);
                free(host);
                continue;
            }

            percent /= 100.0;

            vbox = gtk_vbox_new(FALSE, 0);
            add_cell(vbox, host);

            button = gtk_button_new();
            gtk_signal_connect(GTK_OBJECT(button), "clicked",
                    GTK_SIGNAL_FUNC(host_button_clicked), host);
            gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);

            if (show_host) {
                label = gtk_label_new(host);
                gtk_container_add(GTK_CONTAINER(button), label);

                style = gtk_style_copy(gtk_widget_get_style(label));
                if ((noargs > 1) &&
                        (gdk_color_parse(fg,&(style->fg[GTK_STATE_NORMAL]))!=TRUE)){
                    g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", fg);
                    write_status(status);
                    free(host);
                    continue;
                }
                gtk_widget_set_style(label, style);
                style = gtk_style_copy(gtk_widget_get_style(button));
                if ((noargs > 2) &&
                        (gdk_color_parse(bg,&(style->bg[GTK_STATE_NORMAL]))!=TRUE)){
                    g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", bg);
                    write_status(status);
                    free(host);
                    continue;
                }
                gtk_widget_set_style(button, style);

                progress = gtk_progress_bar_new();
                gtk_progress_bar_update(GTK_PROGRESS_BAR(progress),
                        (gfloat)percent);
                gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(progress), 
                        GTK_PROGRESS_LEFT_TO_RIGHT);
                gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(progress), 
                        GTK_PROGRESS_CONTINUOUS);
                gtk_progress_set_format_string(GTK_PROGRESS(progress), "%p%%");
                gtk_progress_set_show_text(GTK_PROGRESS(progress), TRUE);
                gtk_box_pack_start(GTK_BOX(vbox), progress, TRUE, TRUE, 0);

                gtk_widget_show(progress);
                gtk_widget_show(label);
            } else {
                GdkColor min, max;
                char *tip;

                if (noargs < 3) {
                    g_snprintf(status, BUF_SIZE, 
                            "Percentage given with no host name option, "
                            "but without color range\n");
                    write_status(status);
                    free(host);
                    continue;
                }

                style = gtk_style_copy(gtk_widget_get_style(button));
                if (gdk_color_parse(fg, &min) != TRUE) {
                    g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", fg);
                    write_status(status);
                    free(host);
                    continue;
                }

                if (gdk_color_parse(bg, &max) != TRUE) {
                    g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", bg);
                    write_status(status);
                    free(host);
                    continue;
                }

                style->bg[GTK_STATE_NORMAL].pixel = 0;
                style->bg[GTK_STATE_NORMAL].red = 
                    min.red + (percent * (max.red - min.red));
                style->bg[GTK_STATE_NORMAL].green =
                    min.green + (percent * (max.green - min.green));
                style->bg[GTK_STATE_NORMAL].blue =
                    min.blue + (percent * (max.blue - min.blue));

                gtk_widget_set_style(button, style);

                tip = g_strdup_printf("%s: %g%%", host, percent * 100.0);
                gtk_tooltips_set_tip(button_tips, button, tip, tip);
            }

            gtk_widget_show(button);
        } else if (strcasecmp(fmt, "text") == 0) {
            GtkWidget *vbox, *button, *label;

            vbox = gtk_vbox_new(FALSE, 0);
            add_cell(vbox, host);

            if (show_host) {
                button = gtk_button_new_with_label(host);
            } else {
                button = gtk_button_new();
                gtk_tooltips_set_tip(button_tips, button, host, host);
            }
            gtk_signal_connect(GTK_OBJECT(button), "clicked",
                    GTK_SIGNAL_FUNC(host_button_clicked), host);
            gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);

            label = gtk_label_new(tmp);
            gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

            gtk_widget_show(label);
            gtk_widget_show(button);
        } else if (strcasecmp(fmt, "color") == 0) {
            GtkWidget *button, *label;

            if (sscanf(tmp + argstart, "%s %s", fg, bg) != 2) {
                g_snprintf(status, BUF_SIZE, 
                        "syntax: color <foreground> <background>\n");
                write_status(status);
                free(host);
                continue;
            }

            button = gtk_button_new();
            gtk_signal_connect(GTK_OBJECT(button), "clicked",
                    GTK_SIGNAL_FUNC(host_button_clicked), host);
            add_cell(button, host);

            if (show_host) {
                label = gtk_label_new(host);
                gtk_container_add(GTK_CONTAINER(button), label);
                style = gtk_style_copy(gtk_widget_get_style(label));
                if (gdk_color_parse(fg,&(style->fg[GTK_STATE_NORMAL])) != TRUE){
                    g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", fg);
                    write_status(status);
                    gtk_widget_destroy(button);
                    free(host);
                    continue;
                }
                gtk_widget_set_style(label, style);
                gtk_widget_show(label);
            } else {
                gtk_tooltips_set_tip(button_tips, button, host, host);
            }

            style = gtk_style_copy(gtk_widget_get_style(button));
            if (gdk_color_parse(bg, &(style->bg[GTK_STATE_NORMAL])) != TRUE) {
                g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", bg);
                write_status(status);
                gtk_widget_destroy(button);
                free(host);
                continue;
            }

            gtk_widget_set_style(button, style);
        } else if (sscanf(fmt, "%d", &value) == 1) {
            GtkWidget *button, *label;

            if (value) {
                strcpy(fg, "black");
                strcpy(bg, "green");
            } else {
                strcpy(fg, "black");
                strcpy(bg, "red");
            }

            button = gtk_button_new();
            gtk_signal_connect(GTK_OBJECT(button), "clicked",
                    GTK_SIGNAL_FUNC(host_button_clicked), host);
            add_cell(button, host);

            if (show_host) {
                label = gtk_label_new(host);
                gtk_container_add(GTK_CONTAINER(button), label);
                style = gtk_style_copy(gtk_widget_get_style(label));
                if (gdk_color_parse(fg,&(style->fg[GTK_STATE_NORMAL])) != TRUE){
                    g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", fg);
                    write_status(status);
                    gtk_widget_destroy(button);
                    free(host);
                    continue;
                }
                gtk_widget_set_style(label, style);
                gtk_widget_show(label);
            } else {
                gtk_tooltips_set_tip(button_tips, button, host, host);
            }

            style = gtk_style_copy(gtk_widget_get_style(button));
            if (gdk_color_parse(bg, &(style->bg[GTK_STATE_NORMAL])) != TRUE) {
                g_snprintf(status, BUF_SIZE, "Unknown color '%s'\n", bg);
                write_status(status);
                gtk_widget_destroy(button);
                free(host);
                continue;
            }

            gtk_widget_set_style(button, style);
        } else {
            g_snprintf(status, BUF_SIZE, "Unknown directive: '%s'\n", fmt);
            write_status(status);
            free(host);
        }
    }

    if (feof(f)) {
        gdk_input_remove(input_idx);
        if (input_is_fifo) {
            int fd;

            fclose(f);
            fd = open(fifoname, O_RDONLY|O_NONBLOCK);
            if (fd < 0) {
                g_snprintf(status, BUF_SIZE, "Unable to reopen fifo '%s'\n", fifoname);
                write_status(status);
            } else {
                input_idx = gdk_input_add(fd, GDK_INPUT_READ, handle_input, NULL);
            }
        }
    }
}

void print_usage()
{
    fprintf(stderr,
            "usage: ptdisp [-t <window title>] [-T <window title>] [-h] [-H]\n"
           );
}

int main(int argc, char *argv[])
{
    int c;
    int flags;
    int fd = STDIN_FILENO;
    struct stat st;

    gtk_init(&argc, &argv);

    while ((c = getopt(argc, argv, "t:T:hHpP:")) != -1){
        switch (c) {
            case 't':
            case 'T':
                title = optarg;
                break;
            case 'h':
                show_host = 0;
                break;
            case 'H':
                show_host = 1;
                break;
            case 'p':
                snprintf(fifobuf, 32, "/tmp/ptdisp.%d", getpid());
                fifoname = fifobuf;

                if (mkfifo(fifoname, 0600) != 0) {
                    perror("Unable to make fifo");
                    return 1;
                }
                fd = open(fifoname, O_RDONLY|O_NONBLOCK);
                if (fd < 0) {
                    perror("Unable to open fifo");
                    return 1;
                }
                delete_fifo = 1;
                input_is_fifo = 1;
                break;
            case 'P':
                fifoname = optarg;

                fd = open(fifoname, O_RDONLY|O_NONBLOCK);
                if (fd < 0) {
                    perror("Unable to open fifo");
                    return 1;
                }
                if (stat(fifoname, &st) != 0) {
                    perror("Unable to stat fifo");
                    return 1;
                }
                if (!S_ISFIFO(st.st_mode)) {
                    fprintf(stderr, "%s is not a fifo\n", fifoname);
                    return 1;
                }
                input_is_fifo = 1;
                break;
            case '?':
                print_usage();
                exit(1);
            default:
                break;
        }
    }

    init_mainwindow();

    if (fd == STDIN_FILENO) {
        /* Set stdin to non-blocking */
        flags = fcntl(fd, F_GETFL);
        if (flags == -1) {
            perror("Unable to get flags for standard input");
            return 1;
        }

        flags |= O_NONBLOCK;
        if (fcntl(fd, F_SETFL, flags) == -1){
            perror("Unable to set flags for standard input");
            return 1;
        }
    }

    input_idx = gdk_input_add(fd, GDK_INPUT_READ, handle_input, NULL);

    gtk_main();

    return 0;
}
