/* A wrapper that runs mpirun with the right arguments */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stropts.h>
#include <termio.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "ptools.h"
#include "debug.h"
#include "nodes.h"

#define BUF_SIZE	1024
#define CMD_SIZE	4096

/*N machine_args
+	-m <machine_file> - 
		Specify a file with a list of destination machines.  It is incorrect
		to use this with the -M option.  This option must preceed all others.

.	-M <machine_list> -
		Specify a list of destination machines.  The entire list must be
		surrounded by quotes.  It is incorrect to use this with the -m option.
		This option must preceed all others.
 N*/

/*N machine_envs
-	PT_MACHINE_FILE -
		If PT_MACHINE_FILE is set, the command will attempt to open the file 
		whose name is stored in PT_MACHINE_FILE.  This will be used if neither 
		the -m or -M options are given.
  N*/

int main(int argc, char *argv[])
{
	int i;
    int ret, len;
    int all, optind;
	int num_machines;
	char cmd[CMD_SIZE];
    char *tmpfn;
    FILE *f;

    if (nodes_init(argc, argv, 1, &tmpfn, &num_machines, &all, &optind) < 0)
        return 1;

	/*****************************************/
	/* Get the arguments ready to run mpirun */
	/*****************************************/
    len = snprintf(cmd, CMD_SIZE, "mpirun -np %d%s%s %s \"test ", 
                   all ? num_machines : num_machines,
                   all ? "" : " -m ",
                   all ? "" : tmpfn,
                   PT_MPI);

    for (i = optind; i < argc; i++) {
        len += strlen(argv[i]);
        if ((len < 0) || (len > CMD_SIZE)) {
            fprintf(stderr, "Command too long\n");
            return 1;
        }
        strncat(cmd, argv[i], CMD_SIZE-len);
        
        len += sizeof(" ");
        if ((len < 0) || (len > CMD_SIZE)) {
            fprintf(stderr, "Command too long\n");
            return 1;
        }
        strncat(cmd, " ", CMD_SIZE-len);
    }
    len += sizeof("\"");
    if ((len < 0) || (len > CMD_SIZE)) {
        fprintf(stderr, "Command too long\n");
        return 1;
    }
    strncat(cmd, "\"", CMD_SIZE-len);

    
	/**************/
	/* Run mpirun */
	/**************/
    ret = 1;

    f = popen(cmd, "r");
    if (f == NULL) {
        perror("Execution of mpirun failed");
    } else {
        fscanf(f, "%d", &ret);
    }
    pclose(f);

	/****************************************/
	/* Remove the intermediate machine file */
	/****************************************/
    unlink(tmpfn);

	return ret;
}
