#include "mpi.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "debug.h"
#include "ptwrapper_o.h"

#define MAX_HOSTNAME_LEN	1024
#define BUF_SIZE			1024

#define GLOBAL_RETURN_VAL(s)	\
	(MPI_Allreduce(&(s), &global_status, 1, MPI_INT, MPI_MIN, MPI_COMM_WORLD),\
	 global_status)

int add_hostname = 0;
char cmd[MAX_CMD_LEN];
extern char *progname;
extern int check_args(int argc, char *argv[], char *cmd);

int main(int argc, char *argv[])
{
	int i;
	int rank, csize;
	int status, global_status;
	int count;
	char hostname[MAX_HOSTNAME_LEN];
	char buf[BUF_SIZE];
	MPI_Status mpistat;

    pt_debug("in ptwrapper_o.c, before MPI_Init\n");

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &csize);

	if (rank == 0)
		pt_debug("'%s' in comm of size %d\n", progname, csize);

	status = gethostname(hostname, MAX_HOSTNAME_LEN);
	if (status < 0)
		snprintf(hostname, MAX_HOSTNAME_LEN, "host-%d", rank);
	if (GLOBAL_RETURN_VAL(status) < 0 && rank == 0)
		fprintf(stderr, "Warning: unable to get hostnames of all nodes\n");

	status = check_args(argc, argv, cmd);
	if (GLOBAL_RETURN_VAL(status) < 0){
		if (rank == 0)
			fprintf(stderr, "Invalid arguments\n");
		MPI_Finalize();
		return 1;
	}

	if (rank == 0)
		pt_debug("'%s' starting execution loop\n", progname);

	if (rank == 0){	
		for (i = 1; i < csize; i++){
            if (add_hostname) {
                MPI_Recv(hostname, MAX_HOSTNAME_LEN, MPI_CHAR, 
                         i, 0, MPI_COMM_WORLD, &mpistat);
                printf("[%s]\n", hostname);
            }
			do {
				MPI_Recv(buf, BUF_SIZE, MPI_CHAR, 
                         i, 0, MPI_COMM_WORLD, &mpistat);
				MPI_Get_count(&mpistat, MPI_CHAR, &count);
				if (count != 0)
                    write(STDOUT_FILENO, buf, count);
			} while (count != 0);
		}
	} else {
        int fd;
		FILE *f; 

        if (add_hostname) {
            MPI_Send(hostname, MAX_HOSTNAME_LEN, MPI_CHAR, 
                     0, 0, MPI_COMM_WORLD);
        }
        
        strncat(cmd, " 2>&1", MAX_CMD_LEN-strlen(cmd));
        pt_debug("about to execute '%s'\n", cmd);
        f = popen(cmd, "r");

		if (f == NULL){
			snprintf(buf, BUF_SIZE, "Unable to run %s\n", progname);
			MPI_Send(buf, BUF_SIZE, MPI_CHAR, 0, 0, MPI_COMM_WORLD);
		} else {
            fd = fileno(f);

            while ((count = read(fd, buf, BUF_SIZE)) > 0)
                MPI_Send(buf, count, MPI_CHAR, 0, 0, MPI_COMM_WORLD);
        }
        pclose(f);

		MPI_Send(buf, 0, MPI_CHAR, 0, 0, MPI_COMM_WORLD);
	}

	MPI_Finalize();
	return 0;
}
